#!/usr/bin/env python
# $Id: errors.py 3466 2008-09-26 08:49:01Z hsaliak $
#
# Exceptions.
#
# Copyright 2008 Platform Computing Inc.
#
"""Exception handling follows the SOA model where logical functionality is
grouped together and packaged as cooperating services. With these logical
separations, robust error handling can be introduced by structuring the
exception hierarchy such that each layer(application, command, module)
can inherit from their respective exception superclasses.

By catching the Application, Command, or Module exceptions related to
the code called, the caller can be certain that exceptions raised there
will be caught."""

class PrimitiveException(Exception): pass

class ApplicationException(PrimitiveException): pass
class CommandException(PrimitiveException): pass
class ModuleException(PrimitiveException): pass


# General - legacy stuff kept so as not to break anything
class PrimitiveError(Exception): pass

class InvalidPathError(PrimitiveError): pass
class UnknownTypeError(PrimitiveError): pass
class UnknownFileTypeError(PrimitiveError): pass
class FileDoesNotExistError(PrimitiveError): pass
class FileAlreadyExistError(PrimitiveError): pass
class FileReadPermissionError(PrimitiveError): pass
class DirDoesNotExistError(PrimitiveError): pass
class DirAlreadyExistError(PrimitiveError): pass
class CommandFailedToRunError(PrimitiveError): pass
class UserExitError(PrimitiveError): pass
class UnknownDeviceError(PrimitiveError): pass
class NotImplementedError(PrimitiveError): pass
class UnsupportedURIError(PrimitiveError): pass
class UnsupportedOS(PrimitiveError): pass
class CommandMissingArgsError(PrimitiveError): pass

# Autoinstall exceptions
class TemplateNotFoundException(PrimitiveError): pass
class AutoInstallConfigNotCompleteException(PrimitiveError): pass

# Fetchtool exceptions
class FetchException(PrimitiveError): pass
class HTTPAuthException(PrimitiveError): pass
class UnknownHostException(PrimitiveError): pass
class CommandMissingArgsException(PrimitiveError): pass
class InvalidUserPassPairException(PrimitiveError): pass
class ExistingDestCannotBeOverwrittenException(PrimitiveError): pass
